package com.hypass.export.controller;

import com.google.inject.Inject;
import com.google.inject.servlet.RequestScoped;
import com.hypaas.db.JpaSupport;
import com.hypaas.rpc.Response;
import com.hypass.export.model.*;
import com.hypass.export.service.IExportService;
import java.lang.reflect.InvocationTargetException;
import java.util.List;
import javax.ws.rs.*;
import javax.ws.rs.core.MediaType;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.ObjectUtils;

/**
 * @ClassName InventoryMgrController @Description TODO @Author Will @Date 2023/7/23 18:58 @Version
 * 1.0
 */
@RequestScoped
@Produces(MediaType.APPLICATION_JSON)
@Path("/export")
public class ExportController extends JpaSupport {

  @Inject private IExportService exportService;

  @GET
  @Consumes(MediaType.APPLICATION_JSON)
  @Path("/getTemplateDetail")
  public Response getTemplateDetail(@QueryParam("templateId") String templateId) {
    Response response = new Response();
    ExportTemplateDTO exportTemplateDTO = null;
    try {
      exportTemplateDTO = exportService.getTemplateDetail(templateId);
    } catch (InvocationTargetException e) {
      response.setData(null);
      response.setStatus(Response.STATUS_FAILURE);
    } catch (IllegalAccessException e) {
      response.setData(null);
      response.setStatus(Response.STATUS_FAILURE);
    }
    response.setData(exportTemplateDTO);
    response.setStatus(Response.STATUS_SUCCESS);
    return response;
  }

  @GET
  @Consumes(MediaType.APPLICATION_JSON)
  @Path("/getCascadeModelFieldList")
  public Response getCascadeModelFieldList(
      @QueryParam("modelName") String modelName, @QueryParam("fieldName") String fieldName) {
    Response response = new Response();
    List<CascadeModelFieldDTO> list = exportService.getCascadeModelFieldList(modelName, fieldName);
    response.setData(list);
    response.setStatus(Response.STATUS_SUCCESS);
    return response;
  }

  @POST
  @Consumes(MediaType.APPLICATION_JSON)
  @Path("/saveTemplate")
  public Response saveTemplate(ExportTemplateDTO exportTemplateDTO) {
    Response response = new Response();
    response.setStatus(Response.STATUS_SUCCESS);
    Boolean result = false;
    try {
      result = exportService.saveTemplate(exportTemplateDTO);
    } catch (InvocationTargetException e) {
      result = false;
      response.setStatus(Response.STATUS_FAILURE);
    } catch (IllegalAccessException e) {
      result = false;
      response.setStatus(Response.STATUS_FAILURE);
    }
    response.setData(result);
    return response;
  }

  /**
   * 查询模型列表
   *
   * @return
   */
  @GET
  @Path("/getModelList")
  public Response getModelList() {
    List<MetaModelDTO> modelList = exportService.getModelList();
    Response response = new Response();
    response.setData(modelList);
    response.setStatus(Response.STATUS_SUCCESS);
    return response;
  }

  @GET
  @Consumes(MediaType.APPLICATION_JSON)
  @Path("/preConfigExportStrategy")
  public Response preConfigExportStrategy(
      @QueryParam("modelName") String modelName, @QueryParam("templateId") String templateId) {
    List<ExportStrategyDTO> exportStrategyDTOS = null;
    Response response = new Response();
    exportStrategyDTOS = exportService.preConfigExportStrategy(modelName, templateId);
    response.setData(exportStrategyDTOS);
    response.setStatus(Response.STATUS_SUCCESS);
    return response;
  }

  @GET
  @Consumes(MediaType.APPLICATION_JSON)
  @Path("/getModelExportStrategy")
  public Response getModelExportStrategy(@QueryParam("templateId") String templateId) {
    ExportStrategyDTO modelExportStrategy = null;
    Response response = new Response();
    try {
      modelExportStrategy = exportService.getModelExportStrategy(templateId);
    } catch (InvocationTargetException e) {
      response.setData(null);
      response.setStatus(Response.STATUS_FAILURE);
    } catch (IllegalAccessException e) {
      response.setData(null);
      response.setStatus(Response.STATUS_FAILURE);
    }
    response.setData(modelExportStrategy);
    response.setStatus(Response.STATUS_SUCCESS);
    return response;
  }

  @POST
  @Consumes(MediaType.APPLICATION_JSON)
  @Path("/saveTemplateStrategy")
  public Response saveTemplateStrategy(SaveExportStrategyDTO saveExportStrategyDTO) {
    Boolean result = null;
    Response response = new Response();
    if (ObjectUtils.isEmpty(saveExportStrategyDTO)
        || saveExportStrategyDTO.getTemplateId() == null
        || CollectionUtils.isEmpty(saveExportStrategyDTO.getExportStrategyDTOs())) {
      response.setData(false);
      response.setStatus(Response.STATUS_FAILURE);
      return response;
    }
    result = exportService.saveTemplateStrategy(saveExportStrategyDTO);
    response.setData(result);
    response.setStatus(Response.STATUS_SUCCESS);
    return response;
  }
}
