package com.hypass.export.service.impl;

import com.export.enums.CommonBoolean;
import com.google.inject.Inject;
import com.google.inject.persist.Transactional;
import com.hypaas.db.Query;
import com.hypaas.meta.db.MetaField;
import com.hypaas.meta.db.MetaModel;
import com.hypaas.meta.db.repo.MetaFieldRepository;
import com.hypaas.meta.db.repo.MetaModelRepository;
import com.hypass.export.ExportStrategy;
import com.hypass.export.ExportTemplate;
import com.hypass.export.model.*;
import com.hypass.export.repo.ExportStrategyRepository;
import com.hypass.export.repo.ExportTemplateRepository;
import com.hypass.export.service.IExportService;
import com.hypass.export.utils.HypassBeanUtils;
import java.lang.reflect.InvocationTargetException;
import java.security.InvalidParameterException;
import java.util.ArrayList;
import java.util.List;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;

/**
 * @ClassName ExportServiceImpl @Description TODO @Author wyj @Date 2023/11/18 16:02 @Version 1.0
 */
public class ExportServiceImpl implements IExportService {

  @Inject private MetaModelRepository metaModelRepository;

  @Inject private ExportStrategyRepository exportStrategyRepository;

  @Inject private ExportTemplateRepository exportTemplateRepository;

  @Inject private MetaFieldRepository metaFieldRepository;

  @Override
  public List<MetaModelDTO> getModelList() {
    List<MetaModel> fetch = metaModelRepository.all().fetch();
    if (CollectionUtils.isNotEmpty(fetch)) {
      return HypassBeanUtils.copyProperties4List(fetch, MetaModelDTO.class);
    }
    return null;
  }

  @Override
  public ExportStrategyDTO getModelExportStrategy(String templateId)
      throws InvocationTargetException, IllegalAccessException {
    ExportTemplate exportTemplate = exportTemplateRepository.find(Long.valueOf(templateId));
    ExportStrategy exportStrategy =
        Query.of(ExportStrategy.class)
            .filter("self.templateInfo = :templateInfo")
            .bind("templateInfo", exportTemplate)
            .fetchOne();
    ExportStrategyDTO exportStrategyDTO = new ExportStrategyDTO();
    HypassBeanUtils.copyProperties(exportStrategy, exportStrategyDTO);
    return exportStrategyDTO;
  }

  @Override
  @Transactional
  public boolean saveTemplateStrategy(SaveExportStrategyDTO saveExportStrategyDTO) {
    ExportTemplate exportTemplate =
        exportTemplateRepository.find(Long.valueOf(saveExportStrategyDTO.getTemplateId()));
    if (ObjectUtils.isEmpty(exportTemplate)) {
      throw new RuntimeException("模板不存在");
    }
    saveExportStrategyDTO
        .getExportStrategyDTOs()
        .forEach(
            exportStrategyDTO -> {
              ExportStrategy exportStrategy =
                  exportStrategyRepository.find(exportStrategyDTO.getId());
              exportStrategy.setIsExport(CommonBoolean.valueOf(exportStrategyDTO.getIsExport()));
              exportStrategy.setExportFiledName(exportStrategyDTO.getExportFiledName());
              exportStrategy.setEntityName(exportStrategyDTO.getEntityName());
              exportStrategy.setExportContent(exportStrategyDTO.getExportContent());
              exportStrategy.setFieldAttr(exportStrategyDTO.getFieldAttr());
              exportStrategy.setFieldType(exportStrategyDTO.getFieldType());
              exportStrategy.setFieldName(exportStrategy.getFieldName());
              exportStrategy.setId(exportStrategyDTO.getId());
              exportStrategy.setOutContent(exportStrategy.getOutContent());
              if (exportStrategy.getId() != null) {
                exportStrategyRepository.merge(exportStrategy);
              } else {
                exportStrategyRepository.save(exportStrategy);
              }
            });
    return true;
  }

  @Override
  public List<CascadeModelFieldDTO> getCascadeModelFieldList(String modelName, String fieldName) {
    MetaModel metaModel = metaModelRepository.findByName(modelName);
    MetaField metaField =
        Query.of(MetaField.class)
            .filter("self.metaModel = :metaModel and self.name = :name")
            .bind("metaModel", metaModel)
            .bind("name", fieldName)
            .fetchOne();
    String typeName = metaField.getTypeName();
    MetaModel cascadeModel = metaModelRepository.findByName(typeName);
    List<MetaField> fields =
        Query.of(MetaField.class)
            .filter("self.metaModel = :metaModel")
            .bind("metaModel", cascadeModel)
            .fetch();
    if (CollectionUtils.isNotEmpty(fields)) {
      List<CascadeModelFieldDTO> list = new ArrayList<>();
      fields.forEach(
          field -> {
            CascadeModelFieldDTO cascadeModelDTO = new CascadeModelFieldDTO();
            cascadeModelDTO.setCode(field.getName());
            if (StringUtils.isEmpty(field.getLabel())) {
              cascadeModelDTO.setName(field.getName());
            } else {
              cascadeModelDTO.setName(field.getLabel());
            }
            list.add(cascadeModelDTO);
          });
      return list;
    }
    return null;
  }

  @Override
  public List<ExportStrategyDTO> preConfigExportStrategy(String modelName, String templateId) {
    List<ExportStrategyDTO> exportStrategyDTOS = new ArrayList<>();
    ExportTemplate exportTemplate = exportTemplateRepository.find(Long.valueOf(templateId));
    List<ExportStrategy> templateStrategyInfos =
        Query.of(ExportStrategy.class)
            .filter("self.templateInfo = :templateInfo")
            .bind("templateInfo", exportTemplate)
            .fetch();

    if (CollectionUtils.isEmpty(templateStrategyInfos)) {
      MetaModel metaModel = metaModelRepository.findByName(modelName);
      List<MetaField> fields =
          Query.of(MetaField.class)
              .filter(
                  "self.metaModel = :metaModel and (self.relationship is null or self.relationship='ManyToOne')")
              .bind("metaModel", metaModel)
              .fetch();
      if (CollectionUtils.isEmpty(fields)) {
        return exportStrategyDTOS;
      }
      List<ExportStrategyDTO> finalExportStrategyDTOS = exportStrategyDTOS;
      fields.forEach(
          field -> {
            ExportStrategyDTO exportStrategyDTO = new ExportStrategyDTO();
            exportStrategyDTO.setFieldName(field.getName());
            exportStrategyDTO.setFieldType(field.getTypeName());
            if (StringUtils.isEmpty(field.getRelationship())) {
              exportStrategyDTO.setFieldAttr("普通");
            } else {
              exportStrategyDTO.setFieldAttr(field.getRelationship());
            }
            exportStrategyDTO.setIsExport(CommonBoolean.NOT.getValue());
            finalExportStrategyDTOS.add(exportStrategyDTO);
          });
      return finalExportStrategyDTOS;
    } else {
      return HypassBeanUtils.copyProperties4List(templateStrategyInfos, ExportStrategyDTO.class);
    }
  }

  @Override
  @Transactional
  public boolean saveTemplate(ExportTemplateDTO exportTemplateDTO)
      throws InvocationTargetException, IllegalAccessException {
    if (exportTemplateDTO == null
        || StringUtils.isEmpty(exportTemplateDTO.getEntityName())
        || StringUtils.isEmpty(exportTemplateDTO.getCode())
        || StringUtils.isEmpty(exportTemplateDTO.getName())) {
      throw new InvalidParameterException("参数异常");
    }
    String entityName = exportTemplateDTO.getEntityName();
    MetaModel metaModel = metaModelRepository.findByName(entityName);
    ExportTemplate exportTemplate = new ExportTemplate();
    HypassBeanUtils.copyProperties(exportTemplateDTO, exportTemplate);
    exportTemplate.setEntityName(metaModel);
    exportTemplate.setIsDefaultTemplate(
        CommonBoolean.valueOf(exportTemplateDTO.getIsDefaultTemplate()));
    if (exportTemplateDTO.getIsDefaultTemplate() != null) {
      exportTemplateRepository.save(exportTemplate);
    } else {
      exportTemplateRepository.merge(exportTemplate);
    }
    return true;
  }

  @Override
  public ExportTemplateDTO getTemplateDetail(String templateId)
      throws InvocationTargetException, IllegalAccessException {
    ExportTemplate exportTemplate = exportTemplateRepository.find(Long.valueOf(templateId));
    ExportTemplateDTO exportTemplateDTO = new ExportTemplateDTO();
    MetaModel entityName = exportTemplate.getEntityName();
    exportTemplateDTO.setEntityName(entityName.getName());
    if (exportTemplate.getIsDefaultTemplate() == null) {
      exportTemplateDTO.setIsDefaultTemplate("NOT");
    }
    HypassBeanUtils.copyProperties(exportTemplate, exportTemplateDTO);
    return exportTemplateDTO;
  }
}
